{% extends "page.html" %}

{% block page_banner %}
{% endblock page_banner %}


{% block banner_split %}
{% endblock banner_split %}


{% block page_content %}
{{ super() }}
<div class="container">
    {{ detail }}
</div>
{% endblock page_content %}
