{% extends "base.html" %}

{% macro page_banner() -%}
          <div class="container">
            <div class="col-md-10 mx-auto text-center">
              <h1 class="jumbotron-heading" style="padding: 50px 50px;">
                  {% block index_heading %}
                  Find and explore Python projects with the Simple Repository Browser
                  {% endblock index_heading %}
              </h1>
            </div>

            <form action="{{ url_for('search') }}" type="GET" >

                <div class="col-md-4 mx-auto">
                  <div class="input-group ">
                    <div class="form-outline"  style="width: 100%;">
                      <input type="search" name="query" class="form-control" placeholder="Search projects" />
                    </div>
                  </div>
                </div>

            </form>
            <br><br>
          </div>
{% endmacro %}

{% macro banner_split() %}
        <div class="col-md-8 mx-auto">
          <div class="row">
          </div>
        </div>
{% endmacro %}


{% block page_content %}
        <section class="jumbotron jumbo-banner" id="page-banner">
            <div class="container">
            {{ page_banner() }}
            </div>
        </section>

        <section class="jumbotron jumbo-separator" id="banner-split">
            <div class="container">
            {{ banner_split() }}
            </div>
        </section>

        <section>
            <div class="container">
                <img src="{{ static_file_url('/images/logo-large.svg') }}" class="mx-auto d-block" style="max-height: 40vh;">
            </div>
        </section>
{% endblock page_content %}
