<!DOCTYPE html>
<html>
<head>
    {% block head %}
    <title>{% block title %}Python Simple Repository Browser{% endblock title %}</title>

    <link rel="stylesheet" href={{ static_file_url('/vendored/bootstrap.customized.js.css') }} >
    <link rel="stylesheet" href={{ static_file_url('/css/theme.variables.css') }} >
    <link rel="stylesheet" href={{ static_file_url('/css/theme.css') }} >
    <link rel="icon" href="{{ static_file_url('/images/favicon.ico') }}" type="image/x-icon">

    {% block extra_head %}
    {% endblock extra_head %}
    {% endblock head %}
</head>
  <body class="d-flex flex-column min-vh-100">
  {% block body %}
  <div class="top-nav">
    <div class="container" id="site-header">
      <nav class="navbar navbar-expand-lg navbar-light container" >

      <a class="navbar-brand" href="{{ url_for('index') }}">
          {% block logo %}
            <img src="{{ static_file_url('/images/logo-small.svg') }}" style="height: 80px;">
          {% endblock logo %}
      </a>

      <button class="navbar-toggler" type="button" data-bs-toggle="collapse" data-bs-target="#navbarSupportedContent"
              aria-controls="navbarSupportedContent" aria-expanded="false" aria-label="Toggle navigation">
        <span class="navbar-toggler-icon"></span>
      </button>

      <div class="collapse navbar-collapse" id="navbarSupportedContent">
        <ul class="navbar-nav me-auto mb-2 mb-lg-0">
            {% block navbar_left %}
            {% endblock %}
        </ul>

        <ul class="navbar-nav ms-auto mb-2 mb-lg-0">
          {% block navbar_right %}
            <li class="nav-item">
                <a class="nav-link" href="{{ url_for('about') }}">About</a>
            </li>
          {% endblock %}
        </ul>
      </div>

      </nav>
    </div>
  </div>

      <main role="main" style="min-height: 75vh">
          {% block page_content %}

          {% endblock page_content %}
      </main>

      <footer class="text-muted mt-auto">
      <div class="container text-center">
        <p class="float-right">
            {% block browser_version %}
            simple-repository-browser v{{ browser_version }}
            {% endblock browser_version %}
        </p>
        <p></p>
      </div>
      {% block extra_footer %}
          {# Fontawesome can be used in all templates. #}
          <script src="{{ static_file_url('/vendored/fontawesome-free.all.min.js') }}" defer></script>
      {% endblock extra_footer %}
    </footer>
  {% endblock body %}
  </body>
</html>
