{% extends "base.html" %}
{# Like base, but with a search bar in the top nav -#}

{% block navbar_left %}
        <form action="{{ url_for('search') }}" type="GET">
          <input type="text" name="query" value="{{ search_query if search_query is defined else '' }}" placeholder="Search projects" >
        </form>
{% endblock %}

{% block page_content %}
<div style="padding-top:2em"></div>
{% endblock page_content %}
